//-----------------------------------------------------------------------------
//   File:       FX2_to_extsyncFIFO.c
//   Contents:  
//
//   Copyright (c) 2010 Advantools Semiconductor, All rights reserved
//-----------------------------------------------------------------------------
#pragma NOIV               // Do not generate interrupt vectors

#include "fx2.h"
#include "fx2regs.h"
#include "fx2sdly.h"            // 
                                // 

#include "gpif.h"

#include "ExtFpga.h"

BYTE Configuration;                 // Current configuration
BYTE AlternateSetting;              // Alternate settings
BOOL in_enable = FALSE;             // flag to enable IN transfers
BOOL enum_high_speed = FALSE;       // flag to let firmware know FX2 enumerated at high speed
static WORD xdata Tcount = 0;     // transaction count
static WORD xFIFOBC_IN = 0x0000;  // variable that contains EP6FIFOBCH/L value


void TD_Init(void)             // Called once at startup
{
  // set the CPU clock to 48MHz
  CPUCS = ((CPUCS & ~bmCLKSPD) | bmCLKSPD1);
  SYNCDELAY; 

  EP2CFG = 0xA0;     // EP2OUT, bulk, size 512, 4x buffered
  SYNCDELAY;                         
  EP4CFG = 0x00;     // EP4 not valid
  SYNCDELAY;              
  EP6CFG = 0xE0;     // EP6IN, bulk, size 512, 4x buffered     
  SYNCDELAY;
  EP8CFG = 0x00;     // EP8 not valid
  SYNCDELAY;  

  
  FIFORESET = 0x80;  // set NAKALL bit to NAK all transfers from host
  SYNCDELAY;
  FIFORESET = 0x02;  // reset EP2 FIFO
  SYNCDELAY;
  FIFORESET = 0x06;  // reset EP6 FIFO
  SYNCDELAY;
  FIFORESET = 0x00;  // clear NAKALL bit to resume normal operation
  SYNCDELAY;

  EP2FIFOCFG = 0x01; // allow core to see zero to one transition of auto out bit
  SYNCDELAY;
  EP2FIFOCFG = 0x11; // auto out mode, disable PKTEND zero length send, word ops
  SYNCDELAY;
  EP6FIFOCFG = 0x09; // auto in mode, disable PKTEND zero length send, word ops
  SYNCDELAY; 
  
  GpifInit (); // initialize GPIF registers


  SYNCDELAY;
  EP2GPIFFLGSEL = 0x01; // For EP2OUT, GPIF uses EF flag
  SYNCDELAY;
  EP6GPIFFLGSEL = 0x02; // For EP6IN, GPIF uses FF flag
  SYNCDELAY;

  SYNCDELAY;                  // 
  EP6AUTOINLENH = 0x02;       // set AUTOIN commit length to 512 bytes
  SYNCDELAY;                  // 
  EP6AUTOINLENL = 0x00;
  SYNCDELAY;                  
  enum_high_speed = TRUE;


  PORTCCFG = 0x00;    // [7:0] as PORT
  OEC = 0xFF;         // and as outputs
  PORTECFG &= 0x7F;   // [8] as PORT
  OEE |= 0x80;        // and as output

  ResetFpgaUsbcomm();

}

void TD_Poll(void)
{
  if( GPIFTRIG & 0x80 )               // if GPIF interface IDLE
  {   
    if ( ! ( EP24FIFOFLGS & 0x02 ) )  // if there's a packet in the peripheral domain for EP2
	{
      if ( !(EXTFIFONOTFULL) )           // if the external FIFO is not full // 1==>full  0==>notfull
	  {  


	        SYNCDELAY;    
            GPIFTCB1 = 0x01;                // setup transaction count (512 bytes/2 for word wide -> 0x0100)
            SYNCDELAY;
            GPIFTCB0 = 0x00;
            SYNCDELAY;


		// trigger FIFO write transaction(s), using SFR
        //Setup_FLOWSTATE_Write();
		SYNCDELAY;
		GPIFTRIG = GPIF_EP2;           // launch GPIF FIFO WRITE Transaction from EP2 FIFO
		SYNCDELAY;
  
		SYNCDELAY;
		//while( !( GPIFTRIG & 0x80 ) )     // poll GPIFTRIG.7 GPIF Done bit
		//{
		//	;
		//}
		
		SYNCDELAY;
	  }
    }
  }

  if(in_enable)                             // if IN transfers are enabled
  {
    if ( GPIFTRIG & 0x80 )                  // if GPIF interface IDLE
    { 
      if ( !(EXTFIFONOTEMPTY) )                // if external FIFO is not empty // 1==> empty   0 ==>notempty
	  {
	    if ( !( EP68FIFOFLGS & 0x01 ) )     // if EP6 FIFO is not full
		{      

	        SYNCDELAY;    
            GPIFTCB1 = 0x01;                // setup transaction count (512 bytes/2 for word wide -> 0x0100)
            SYNCDELAY;
            GPIFTCB0 = 0x00;
            SYNCDELAY;

  
          //Setup_FLOWSTATE_Read();
          SYNCDELAY;
		  GPIFTRIG = GPIFTRIGRD | GPIF_EP6; // launch GPIF FIFO READ Transaction to EP6 FIFO
	      SYNCDELAY;

	      while( !( GPIFTRIG & 0x80 ) )     // poll GPIFTRIG.7 GPIF Done bit
          {
            ;
          }
    
	      SYNCDELAY;
			
		  //in_enable =0;

		}
	  }
    }
  }

}

BOOL TD_Suspend(void)          // Called before the device goes into suspend mode
{
   return(TRUE);
}

BOOL TD_Resume(void)          // Called after the device resumes
{
   return(TRUE);
}

//-----------------------------------------------------------------------------
// Device Request hooks
//   The following hooks are called by the end point 0 device request parser.
//-----------------------------------------------------------------------------

BOOL DR_GetDescriptor(void)
{
   return(TRUE);
}

BOOL DR_SetConfiguration(void)   // Called when a Set Configuration command is received
{
  if( EZUSB_HIGHSPEED( ) )
  { // FX2 enumerated at high speed
    SYNCDELAY;                  // 
    EP6AUTOINLENH = 0x02;       // set AUTOIN commit length to 512 bytes
    SYNCDELAY;                  // 
    EP6AUTOINLENL = 0x00;
    SYNCDELAY;                  
    enum_high_speed = TRUE;
  }
  else
  { // FX2 enumerated at full speed
    SYNCDELAY;                   
    EP6AUTOINLENH = 0x00;       // set AUTOIN commit length to 64 bytes
    SYNCDELAY;                   
    EP6AUTOINLENL = 0x40;
    SYNCDELAY;                  
    enum_high_speed = FALSE;
  }

  Configuration = SETUPDAT[2];
  return(TRUE);            // Handled by user code
}

BOOL DR_GetConfiguration(void)   // Called when a Get Configuration command is received
{
   EP0BUF[0] = Configuration;
   EP0BCH = 0;
   EP0BCL = 1;
   return(TRUE);            // Handled by user code
}

BOOL DR_SetInterface(void)       // Called when a Set Interface command is received
{
   AlternateSetting = SETUPDAT[2];
   return(TRUE);            // Handled by user code
}

BOOL DR_GetInterface(void)       // Called when a Set Interface command is received
{
   EP0BUF[0] = AlternateSetting;
   EP0BCH = 0;
   EP0BCL = 1;
   return(TRUE);            // Handled by user code
}

BOOL DR_GetStatus(void)
{
   return(TRUE);
}

BOOL DR_ClearFeature(void)
{
   return(TRUE);
}

BOOL DR_SetFeature(void)
{
   return(TRUE);
}

#define VX_B2 0xB2 // reset FPGA
#define VX_B3 0xB3 // enable IN transfers
#define VX_B4 0xB4 // disable IN transfers
#define VX_B5 0xB5 // set fpga reg addr
#define VX_B6 0xB6 // write fpga register
#define VX_B7 0xB7 // read  fpga register
#define VX_B8 0xB8 // read  fpga register
#define VX_B9 0xB9 // read  fpga register
#define VX_BA 0xBA // read  fpga register
#define VX_BB 0xBB // read  fpga register
#define VX_BC 0xBC // read  fpga register
#define VX_BD 0xBD // read  fpga register

BOOL DR_VendorCmnd(void)
{
  WORD xdata buf;

  switch (SETUPDAT[1])
  {
    case VX_B2:
    { 
      // reset FPGA
	  	
	  ResetFpgaUsbcomm();

      EP0BUF[0] = VX_B2;
	  EP0BCH = 0;
	  EP0BCL = 1;                   // Arm endpoint with # bytes to transfer
	  EP0CS |= bmHSNAK;             // Acknowledge handshake phase of device request
      break;
    }
	case VX_B3: // enable IN transfers
	{
	  in_enable = TRUE;

      EP0BUF[0] = VX_B3;
	  EP0BUF[1] = in_enable;
  	  EP0BCH = 0;
	  EP0BCL = 2;
	  EP0CS |= bmHSNAK;
	  break;
    }
	case VX_B4: // disable IN transfers
	{
	  in_enable = FALSE;

      EP0BUF[0] = VX_B4;
	  EP0BUF[1] = in_enable;
  	  EP0BCH = 0;
	  EP0BCL = 2;
	  EP0CS |= bmHSNAK;
	  break;
    }
	case VX_B5: // set fpga reg addr
	{	
	  buf =	SETUPDAT[3];
	  buf <<=8;
	  buf |=SETUPDAT[2];
		
	  SetFpgaRegAddr(buf);

	  EP0BUF[0] = VX_B5;
	  SYNCDELAY;
	  EP0BUF[1] = buf>>8;
	  SYNCDELAY;
	  EP0BUF[2] = buf;
 	  SYNCDELAY;

  	  EP0BCH = 0;
	  EP0BCL = 3;
	  EP0CS |= bmHSNAK;
	  break;
    }
    case VX_B6: // write fpga register
	{	
	  buf =	SETUPDAT[3];
	  buf <<=8;
	  buf |=SETUPDAT[2];
	  	
	  WriteFpgaRegValue(buf);
	
	  EP0BUF[0] = VX_B6;
	  SYNCDELAY;
	  EP0BUF[1] = buf>>8;
	  SYNCDELAY;
	  EP0BUF[2] = buf;
 	  SYNCDELAY;	

  	  EP0BCH = 0;
	  EP0BCL = 3;
	  EP0CS |= bmHSNAK;
	  break;
    }
    case VX_B7: // read  fpga register
	{
		  
      ReadFpgaRegValue(&buf);
	  
	  EP0BUF[0] = VX_B7;
	  SYNCDELAY;
	  EP0BUF[2] = buf>>8;
	  SYNCDELAY;
	  EP0BUF[1] = buf;
 	  SYNCDELAY;

  	  EP0BCH = 0;
	  EP0BCL = 3;
	  EP0CS |= bmHSNAK;
	  break;
    }
    case VX_B8: // λUSB_FIFO 
	{
	  IOC =0x01;
	  
	  ResetFpgaUsbcomm();

	  IOC =0x00; //ʹUTMڸλ״̬
	 
	  
	  EP0BUF[0] = VX_B8;
	  SYNCDELAY;

  	  EP0BCH = 0;
	  EP0BCL = 1;
	  EP0CS |= bmHSNAK;
	  break;
    }
    case VX_B9: // ʹutmд
	{
	  IOC =0x02; // utm_rden =0 utm_wren =1 rst_utm =0;
 
	  EP0BUF[0] = VX_B9;
	  SYNCDELAY;

  	  EP0BCH = 0;
	  EP0BCL = 1;
	  EP0CS |= bmHSNAK;
	  break;
    }
    case VX_BA: // ֹutmд
	{
	  IOC =0x00; // utm_rden =0 utm_wren =0 rst_utm =0;
 
	  EP0BUF[0] = VX_BA;
	  SYNCDELAY;

  	  EP0BCH = 0;
	  EP0BCL = 1;
	  EP0CS |= bmHSNAK;
	  break;
    }
     case VX_BB: // ʹutm
	{
	  IOC =0x04; //    utm_rden =1 utm_wren =0 rst_utm =0;
 
	  EP0BUF[0] = VX_BB;
	  SYNCDELAY;

  	  EP0BCH = 0;
	  EP0BCL = 1;
	  EP0CS |= bmHSNAK;
	  break;
    }
    case VX_BC: // ֹutm
	{
	  IOC =0x00; // utm_rden =0 utm_wren =0 rst_utm =0;
 
	  EP0BUF[0] = VX_BC;
	  SYNCDELAY;

  	  EP0BCH = 0;
	  EP0BCL = 1;
	  EP0CS |= bmHSNAK;
	  break;
    }

	default:
        return(TRUE);
  }

  return(FALSE);
}

//-----------------------------------------------------------------------------
// USB Interrupt Handlers
//   The following functions are called by the USB interrupt jump table.
//-----------------------------------------------------------------------------

// Setup Data Available Interrupt Handler
void ISR_Sudav(void) interrupt 0
{
   GotSUD = TRUE;            // Set flag
   EZUSB_IRQ_CLEAR();
   USBIRQ = bmSUDAV;         // Clear SUDAV IRQ
}

// Setup Token Interrupt Handler
void ISR_Sutok(void) interrupt 0
{
   EZUSB_IRQ_CLEAR();
   USBIRQ = bmSUTOK;         // Clear SUTOK IRQ
}

void ISR_Sof(void) interrupt 0
{
   EZUSB_IRQ_CLEAR();
   USBIRQ = bmSOF;            // Clear SOF IRQ
}

void ISR_Ures(void) interrupt 0
{
   // whenever we get a USB reset, we should revert to full speed mode
   pConfigDscr = pFullSpeedConfigDscr;
   ((CONFIGDSCR xdata *) pConfigDscr)->type = CONFIG_DSCR;
   pOtherConfigDscr = pHighSpeedConfigDscr;
   ((CONFIGDSCR xdata *) pOtherConfigDscr)->type = OTHERSPEED_DSCR;

   EZUSB_IRQ_CLEAR();
   USBIRQ = bmURES;         // Clear URES IRQ
}

void ISR_Susp(void) interrupt 0
{
   Sleep = TRUE;
   EZUSB_IRQ_CLEAR();
   USBIRQ = bmSUSP;
}

void ISR_Highspeed(void) interrupt 0
{
   if (EZUSB_HIGHSPEED())
   {
      pConfigDscr = pHighSpeedConfigDscr;
      ((CONFIGDSCR xdata *) pConfigDscr)->type = CONFIG_DSCR;
      pOtherConfigDscr = pFullSpeedConfigDscr;
      ((CONFIGDSCR xdata *) pOtherConfigDscr)->type = OTHERSPEED_DSCR;
   }

   EZUSB_IRQ_CLEAR();
   USBIRQ = bmHSGRANT;
}
void ISR_Ep0ack(void) interrupt 0
{
}
void ISR_Stub(void) interrupt 0
{
}
void ISR_Ep0in(void) interrupt 0
{
}
void ISR_Ep0out(void) interrupt 0
{
}
void ISR_Ep1in(void) interrupt 0
{
}
void ISR_Ep1out(void) interrupt 0
{
}
void ISR_Ep2inout(void) interrupt 0
{
}
void ISR_Ep4inout(void) interrupt 0
{
}
void ISR_Ep6inout(void) interrupt 0
{
}
void ISR_Ep8inout(void) interrupt 0
{
}
void ISR_Ibn(void) interrupt 0
{
}
void ISR_Ep0pingnak(void) interrupt 0
{
}
void ISR_Ep1pingnak(void) interrupt 0
{
}
void ISR_Ep2pingnak(void) interrupt 0
{
}
void ISR_Ep4pingnak(void) interrupt 0
{
}
void ISR_Ep6pingnak(void) interrupt 0
{
}
void ISR_Ep8pingnak(void) interrupt 0
{
}
void ISR_Errorlimit(void) interrupt 0
{
}
void ISR_Ep2piderror(void) interrupt 0
{
}
void ISR_Ep4piderror(void) interrupt 0
{
}
void ISR_Ep6piderror(void) interrupt 0
{
}
void ISR_Ep8piderror(void) interrupt 0
{
}
void ISR_Ep2pflag(void) interrupt 0
{
}
void ISR_Ep4pflag(void) interrupt 0
{
}
void ISR_Ep6pflag(void) interrupt 0
{
}
void ISR_Ep8pflag(void) interrupt 0
{
}
void ISR_Ep2eflag(void) interrupt 0
{
}
void ISR_Ep4eflag(void) interrupt 0
{
}
void ISR_Ep6eflag(void) interrupt 0
{
}
void ISR_Ep8eflag(void) interrupt 0
{
}
void ISR_Ep2fflag(void) interrupt 0
{
}
void ISR_Ep4fflag(void) interrupt 0
{
}
void ISR_Ep6fflag(void) interrupt 0
{
}
void ISR_Ep8fflag(void) interrupt 0
{
}
void ISR_GpifComplete(void) interrupt 0
{
}
void ISR_GpifWaveform(void) interrupt 0
{
}


void ResetFpgaUsbcomm( void )
{
  IOC = 0x01;  //ֹDDR2д (debug) ʹutmλ ʹ䴦ڸλ״̬
  IOE = 0x00;  //
  
  OEA |= 0x20;  //PA5 output
  
  IOA |= 0x20;  //1
  SYNCDELAY;
  SYNCDELAY;
  SYNCDELAY;
  IOA &= ~0x20; //0
  SYNCDELAY;
  SYNCDELAY;
  SYNCDELAY;
  IOA |= 0x20;  //1

}
/*************************************************************
func void SetFpgaRegAddr( WORD addr )
desc Set FPGA REG's address
input WORD addr :FPGA REG's address
output --
*************************************************************/
void SetFpgaRegAddr( WORD addr )
{

	IOE =0x80; //reg_aden =1

    Peripheral_SingleWordWrite(addr);

}
/*************************************************************
void SetFpgaRegValue( WORD value )
desc Set the FPGA REG's value
input WORD addr :FPGA REG's value
output --
*************************************************************/
void WriteFpgaRegValue( WORD value )
{
	IOE =0x00; //reg_aden =0

    Peripheral_SingleWordWrite(value);

}
/*************************************************************
void SetFpgaRegValue( WORD value )
desc Set the FPGA REG's value
input WORD addr :FPGA REG's value
output --
*************************************************************/
void ReadFpgaRegValue( WORD *value )
{
	IOE =0x00; //reg_aden =0

    Peripheral_SingleWordRead(value);

}
